@@ -11,15 +11,13 @@
    Refer to licensing information at http://www.artifex.com or contact
    Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
    CA  94903, U.S.A., +1(415)492-9861, for further information.
-*/
+ */
 
 
 /* XPS interpreter - text drawing support */
 
 #include "ghostxps.h"
 
-#include <ctype.h>
-
 #define XPS_TEXT_BUFFER_SIZE 300
 
 typedef struct xps_text_buffer_s xps_text_buffer_t;
@@ -32,11 +30,13 @@ struct xps_text_buffer_s
     gs_glyph g[XPS_TEXT_BUFFER_SIZE];
 };
 
-static inline int unhex(int i)
+static inline int unhex(int c)
 {
-    if (isdigit(i))
-        return i - '0';
-    return tolower(i) - 'a' + 10;
+    if (c >= '0' && c <= '9')
+        return c - '0';
+    if (c >= 'A' && c <= 'Z')
+        return c - 'A' + 10;
+    return c - 'a' + 10;
 }
 
 void
@@ -129,7 +129,6 @@ xps_select_best_font_encoding(xps_font_t *font)
         { 3, 3 },       /* Prc */
         { 3, 2 },       /* ShiftJis */
         { 3, 0 },       /* Symbol */
-        // { 0, * }, -- Unicode (deprecated)
         { 1, 0 },
         { -1, -1 },
     };
@@ -385,7 +384,7 @@ xps_parse_glyphs_imp(xps_context_t *ctx, xps_font_t *font, float size,
             /* Adjust glyph offset and advance width for emboldening */
             if (sim_bold)
             {
-                advance *= 1.02;
+                advance *= 1.02f;
                 u_offset += 0.01 * size;
                 v_offset += 0.01 * size;
             }
@@ -435,7 +434,7 @@ xps_parse_glyphs(xps_context_t *ctx,
     char *opacity_mask_uri;
 
     char *bidi_level_att;
-    char *caret_stops_att;
+    /*char *caret_stops_att;*/
     char *fill_att;
     char *font_size_att;
     char *font_uri_att;
@@ -472,14 +471,14 @@ xps_parse_glyphs(xps_context_t *ctx,
     int sim_bold = 0;
     int sim_italic = 0;
 
-    gs_matrix shear = { 1, 0, 0.36397, 1, 0, 0 }; /* shear by 20 degrees */
+    gs_matrix shear = { 1, 0, 0.36397f, 1, 0, 0 }; /* shear by 20 degrees */
 
     /*
      * Extract attributes and extended attributes.
      */
 
     bidi_level_att = xps_att(root, "BidiLevel");
-    caret_stops_att = xps_att(root, "CaretStops");
+    /*caret_stops_att = xps_att(root, "CaretStops");*/
     fill_att = xps_att(root, "Fill");
     font_size_att = xps_att(root, "FontRenderingEmSize");
     font_uri_att = xps_att(root, "FontUri");
@@ -641,12 +640,12 @@ xps_parse_glyphs(xps_context_t *ctx,
 
     if (fill_att)
     {
-        float samples[32];
+        float samples[XPS_MAX_COLORS];
         gs_color_space *colorspace;
 
         xps_parse_color(ctx, base_uri, fill_att, &colorspace, samples);
         if (fill_opacity_att)
-            samples[0] = atof(fill_opacity_att);
+            samples[0] *= atof(fill_opacity_att);
         xps_set_color(ctx, colorspace, samples);
 
         if (sim_bold)
