@@ -32,6 +32,7 @@ struct stop
 {
     float offset;
     float color[4];
+    int index;
 };
 
 static int cmp_stop(const void *a, const void *b)
@@ -43,7 +44,7 @@ static int cmp_stop(const void *a, const void *b)
         return -1;
     if (diff > 0)
         return 1;
-    return 0;
+    return astop->index - bstop->index;
 }
 
 static inline float lerp(float a, float b, float x)
@@ -77,6 +78,7 @@ xps_parse_gradient_stops(xps_context_t *ctx, char *base_uri, xps_item_t *node,
             if (offset && color)
             {
                 stops[count].offset = atof(offset);
+                stops[count].index = count;
 
                 xps_parse_color(ctx, base_uri, color, &colorspace, sample);
 
@@ -98,7 +100,7 @@ xps_parse_gradient_stops(xps_context_t *ctx, char *base_uri, xps_item_t *node,
                     int num_colors = gsicc_getsrc_channel_count(colorspace->cmm_icc_profile_data);
                     for (i = 0; i < num_colors; i++)
                     {
-                        sample_in[i] = sample[i+1]*65535;
+                        sample_in[i] = (unsigned short)(sample[i+1]*65535);
                     }
                     gscms_transform_color((gx_device *)(ctx->pgs->device),
                                           icclink, sample_in, sample_out, 2);
@@ -259,12 +261,20 @@ xps_create_gradient_stop_function(xps_context_t *ctx, struct stop *stops, int co
     k = count - 1; /* number of intervals / functions */
 
     domain = xps_alloc(ctx, 2 * sizeof(float));
+    if (!domain) {
+        gs_throw(gs_error_VMerror, "out of memory: domain\n");
+        return NULL;
+    }
     domain[0] = 0.0;
     domain[1] = 1.0;
     sparams.m = 1;
     sparams.Domain = domain;
 
     range = xps_alloc(ctx, 6 * sizeof(float));
+    if (!range) {
+        gs_throw(gs_error_VMerror, "out of memory: range\n");
+        return NULL;
+    }
     range[0] = 0.0;
     range[1] = 1.0;
     range[2] = 0.0;
@@ -274,8 +284,20 @@ xps_create_gradient_stop_function(xps_context_t *ctx, struct stop *stops, int co
     sparams.Range = range;
 
     functions = xps_alloc(ctx, k * sizeof(void*));
+    if (!functions) {
+        gs_throw(gs_error_VMerror, "out of memory: functions.\n");
+        return NULL;
+    }
     bounds = xps_alloc(ctx, (k - 1) * sizeof(float));
+    if (!bounds) {
+        gs_throw(gs_error_VMerror, "out of memory: bounds.\n");
+        return NULL;
+    }
     encode = xps_alloc(ctx, (k * 2) * sizeof(float));
+    if (!encode) {
+        gs_throw(gs_error_VMerror, "out of memory: encode.\n");
+        return NULL;
+    }
 
     sparams.k = k;
     sparams.Functions = functions;
@@ -296,12 +318,20 @@ xps_create_gradient_stop_function(xps_context_t *ctx, struct stop *stops, int co
     for (i = 0; i < k; i++)
     {
         domain = xps_alloc(ctx, 2 * sizeof(float));
+        if (!domain) {
+            gs_throw(gs_error_VMerror, "out of memory: domain.\n");
+            return NULL;
+    }
         domain[0] = 0.0;
         domain[1] = 1.0;
         lparams.m = 1;
         lparams.Domain = domain;
 
         range = xps_alloc(ctx, 6 * sizeof(float));
+        if (!range) {
+            gs_throw(gs_error_VMerror, "out of memory: range.\n");
+            return NULL;
+        }
         range[0] = 0.0;
         range[1] = 1.0;
         range[2] = 0.0;
@@ -311,9 +341,17 @@ xps_create_gradient_stop_function(xps_context_t *ctx, struct stop *stops, int co
         lparams.Range = range;
 
         c0 = xps_alloc(ctx, 3 * sizeof(float));
+        if (!c0) {
+            gs_throw(gs_error_VMerror, "out of memory: c0.\n");
+            return NULL;
+        }
         lparams.C0 = c0;
 
         c1 = xps_alloc(ctx, 3 * sizeof(float));
+        if (!c0) {
+            gs_throw(gs_error_VMerror, "out of memory: c1.\n");
+            return NULL;
+        }
         lparams.C1 = c1;
 
         if (opacity_only)
@@ -567,7 +605,7 @@ xps_draw_one_linear_gradient(xps_context_t *ctx,
  * reach a reasonable limit for infinite cases.
  */
 
-static inline float point_inside_circle(float px, float py, float x, float y, float r)
+static inline int point_inside_circle(float px, float py, float x, float y, float r)
 {
     float dx = px - x;
     float dy = py - y;
@@ -578,8 +616,8 @@ static int
 xps_draw_radial_gradient(xps_context_t *ctx, xps_item_t *root, int spread, gs_function_t *func)
 {
     gs_rect bbox;
-    float x0, y0, r0;
-    float x1, y1, r1;
+    float x0 = 0, y0 = 0, r0;
+    float x1 = 0, y1 = 0, r1;
     float xrad = 1;
     float yrad = 1;
     float invscale;
@@ -602,13 +640,18 @@ xps_draw_radial_gradient(xps_context_t *ctx, xps_item_t *root, int spread, gs_fu
     if (radius_y_att)
         yrad = atof(radius_y_att);
 
-    /* scale the ctm to make ellipses */
     gs_gsave(ctx->pgs);
-    gs_scale(ctx->pgs, 1.0, yrad / xrad);
 
-    invscale = xrad / yrad;
-    y0 = y0 * invscale;
-    y1 = y1 * invscale;
+    /* scale the ctm to make ellipses */
+    if (xrad != 0)
+        gs_scale(ctx->pgs, 1.0, yrad / xrad);
+
+    if (yrad != 0)
+    {
+        invscale = xrad / yrad;
+        y0 = y0 * invscale;
+        y1 = y1 * invscale;
+    }
 
     r0 = 0.0;
     r1 = xrad;
@@ -797,8 +840,8 @@ xps_draw_linear_gradient(xps_context_t *ctx, xps_item_t *root, int spread, gs_fu
             if (dist[i] > d1) d1 = dist[i];
         }
 
-        i0 = floor(d0 / len);
-        i1 = ceil(d1 / len);
+        i0 = (int)floor(d0 / len);
+        i1 = (int)ceil(d1 / len);
 
         for (i = i0; i < i1; i++)
         {
@@ -834,9 +877,9 @@ xps_parse_gradient_brush(xps_context_t *ctx, char *base_uri, xps_resource_t *dic
     xps_item_t *node;
 
     char *opacity_att;
-    char *interpolation_att;
+    /*char *interpolation_att;*/
     char *spread_att;
-    char *mapping_att;
+    /*char *mapping_att;*/
     char *transform_att;
 
     xps_item_t *transform_tag = NULL;
@@ -855,9 +898,9 @@ xps_parse_gradient_brush(xps_context_t *ctx, char *base_uri, xps_resource_t *dic
     int has_opacity = 0;
 
     opacity_att = xps_att(root, "Opacity");
-    interpolation_att = xps_att(root, "ColorInterpolationMode");
+    /*interpolation_att = xps_att(root, "ColorInterpolationMode");*/
     spread_att = xps_att(root, "SpreadMethod");
-    mapping_att = xps_att(root, "MappingMode");
+    /*mapping_att = xps_att(root, "MappingMode");*/
     transform_att = xps_att(root, "Transform");
 
     for (node = xps_down(root); node; node = xps_next(node))
